<?php
/*
 * This file belongs to the YIT Framework.
 *
 * This source file is subject to the GNU GENERAL PUBLIC LICENSE (GPL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.gnu.org/licenses/gpl-3.0.txt
 */
if ( !defined( 'YITH_WCDPPM_VERSION' ) ) {
    exit( 'Direct access forbidden.' );
}

/**
 *
 *
 * @class      YITH_Dynamic_Pricing_Payments_Methods_Admin_Premium
 * @package    Yithemes
 * @since      Version 1.0.0
 * @author     Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
 *
 */

if ( !class_exists( 'YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium' ) ) {
    /**
     * Class YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium
     *
     * @author Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
     */
    class YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium extends YITH_Dynamic_Pricing_Payments_Methods_Frontend
    {
        /**
         * Single instance of the class
         *
         * @var \YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium
         * @since 1.0.0
         */
        protected static $_instance;


        /**
         * Returns single instance of the class
         *
         * @return \YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium
         * @since 1.0.0
         */
        public static function get_instance()
        {
            $self = __CLASS__ . ( class_exists( __CLASS__ . '_Premium' ) ? '_Premium' : '' );

            if ( is_null( $self::$_instance ) ) {
                $self::$_instance = new $self;
            }

            return $self::$_instance;
        }

        /**
         * Construct
         *
         * @author Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
         * @since 1.0
         */
        public function __construct()
        {
            add_filter('woocommerce_gateway_description',array($this,'add_gateway_description'),15,2);
            if (version_compare(WC()->version, '3.0.0', '>=')) {
                add_filter('woocommerce_get_formatted_order_total',array($this,'override_total'),10,4);
                add_filter('woocommerce_get_order_item_totals',array($this,'override_tax_field'),15,3);

            } else {
                add_filter('woocommerce_get_formatted_order_total',array($this,'override_total'),10,2);
                add_filter('woocommerce_get_order_item_totals',array($this,'override_tax_field'),15,2);
            }


            parent::__construct();
        }

        /**
         * Add gateway title
         * @access public
         * @return string
         * @since 1.0.0
         * @author   Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
         */
        public function add_gateway_title($title,$type) {

            $amount = 0; //prevent issues with apply_filters
            if (version_compare(WC()->version, '3.0.2', '<=')) {
                if (is_checkout() && !is_account_page() && !did_action('woocommerce_checkout_process')) {
                    $functions = YITH_Dynamic_Pricing_Payments_Methods()->functions;
                    $user = wp_get_current_user();

                    $amount = $functions->yith_wcdppm_get_amount($type, $user->roles, true);
                    if ($amount[0] != 0) {
                        if ($amount[0] > 0) {
                            $title = $title . apply_filters('yith_wcdppm_add_tax_in_amount', ' +' . wc_price($amount[0]), $amount[0], $amount[1]);

                        } else {
                            $title = $title . apply_filters('yith_wcdppm_add_tax_in_amount', ' ' . wc_price($amount[0]), $amount[0], $amount[1]);
                        }
                    }
                }
            } else {
                if ( !is_account_page() && !did_action('woocommerce_checkout_process')) {

                    $functions = YITH_Dynamic_Pricing_Payments_Methods()->functions;
                    $user = wp_get_current_user();

                    $amount = $functions->yith_wcdppm_get_amount($type, $user->roles, true);
                    if ($amount[0] != 0) {
                        if ($amount[0] > 0) {
                            $title = $title . apply_filters('yith_wcdppm_add_tax_in_amount', ' +' . wc_price($amount[0]), $amount[0], $amount[1]);

                        } else {
                            $title = $title . apply_filters('yith_wcdppm_add_tax_in_amount', ' ' . wc_price($amount[0]), $amount[0], $amount[1]);
                        }
                    }
                }
            }

            return apply_filters('yith_wcdppm_gateway_title', $title, $type,$amount);
        }

        /**
         * Add gateway description
         * @access public
         * @since 1.0.0
         * @author   Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
         */

        public function add_gateway_description($description,$type) {
            $functions = YITH_Dynamic_Pricing_Payments_Methods()->functions;
            $user = wp_get_current_user();
            $desc = $functions->yith_wcdppm_get_desc($type,$user->roles,$description);
            if ($desc && !is_account_page()){
                return $desc;
            }
            return $description;
        }

        /**
         * Add payment method row to an order details before place order
         * @access public
         * @since 1.0.0
         * @author   Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
         */
        public function add_row_payment_method() {

            if( !is_account_page() ) {
                $functions = YITH_Dynamic_Pricing_Payments_Methods()->functions;
                $user = wp_get_current_user();
                $gateways = $functions->yith_wcdppm_get_payment_gateway();
                $total = WC()->cart->total;
                foreach ($gateways as $gateway => $value) {
                    $amount = $functions->yith_wcdppm_get_amount($gateway, $user->roles, true);
                    $new_total = $total + $amount[0] + $amount[1];
                    $taxes = WC()->cart->get_taxes();
                    $first_taxes = !empty($taxes) ? $taxes[array_keys($taxes)[0]] : 0;
                    $total_json = array(
                        'amount' => wc_tax_enabled() ? apply_filters('yith_wcdppm_add_tax_in_amount_json',wc_price($amount[0]),$amount[0],$amount[1]) : wc_price($amount[0]),
                        'total' => $this->get_total_row_payment_method( wc_price($new_total),$amount[1] ),
                        'free' => wc_price(0),
                        'tax' => wc_tax_enabled() ? wc_price($first_taxes + $amount[1]) : wc_price(0),
                    );
                    $data_to_file_json = json_encode($total_json);
                    ?>
                    <input type="hidden" id="yith-wcdppm-gateway-<?php echo $gateway ?>"
                           data-total="<?php echo $new_total ?>" data-amount="<?php echo $amount[0] ?>"
                           data-tax_amount="<?php echo $amount[1] ?>"
                           data-payment-method="<?php echo htmlspecialchars($data_to_file_json) ?>">
                    <?php
                }
                ?>
                <tr class="yith-wcdppm-payment-method">
                    <th><?php echo apply_filters('yith_wcdppm_change_name_payment_method', __('Payment method', 'yith-dynamic-pricing-per-payment-method-for-woocommerce')); ?></th>
                    <td id="yith-wcdppm-amount">

                    </td>
                </tr>
                <?php
            }
        }

        public function add_item_fee($order_id) {
            if( !is_account_page() ) {
                $functions = YITH_Dynamic_Pricing_Payments_Methods()->functions;
                $user = wp_get_current_user();
                $order = wc_get_order($order_id);
                $payment_method = yit_get_prop($order, '_payment_method', true);

                $exist_payment = $functions->exist_payment($payment_method, $user->roles);
                if(!$exist_payment){
                    return;
                }

                $amount = $functions->yith_wcdppm_get_amount($payment_method, $user->roles, true);
                yit_save_prop($order, 'yith_wcdppm_amount', $amount);

                $tax_class = $functions->get_tax_class_payment_rule($payment_method,$user->roles);

                $order_fee_id = 'yith-wcdppm_fee';

                if ($amount[0] >= 0) {
                    $order_fee_title = __('Payment Method Fee', 'yith-dynamic-pricing-per-payment-method-for-woocommerce');
                } else {
                    $order_fee_title = __('Payment Method Discount', 'yith-dynamic-pricing-per-payment-method-for-woocommerce');
                }
                if ($order instanceof WC_Data) {

                    $item = new WC_Order_Item_Fee();

                    $error = $item->set_props(array(
                        'name' => apply_filters('yith_wcdppm_order_fee_title',$order_fee_title,$amount),
                        'tax_class' => $tax_class,
                        'tax_status' => empty($tax_class) ? 'none' : '',
                        'total' => floatval($amount[0]),
                        'order_id' => $order_id,
                    ));
                    $item->set_taxes(array('total' => array(1 => $amount[1])));
                    $item->save();
                    $order->update_taxes();
                    $order->calculate_totals();
                    $order->save();

                } else {

                    $order_fee_amount = $amount[0];
                    $order_fee = new stdClass();
                    $order_fee->id = $order_fee_id;
                    $order_fee->name = $order_fee_title;
                    $order_fee->amount = floatval($order_fee_amount);
                    $order_fee->taxable = false;
                    $order_fee->tax = floatval($amount[1]);
                    $order_fee->tax_data = array();
                    $order_fee->tax_class = $tax_class;

                    $order->add_fee($order_fee);
                }
            }
        }

        /**
         * Add metadata amount to an order and set new total order
         * @access public
         * @since 1.0.0
         * @author   Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
         */
        public function add_amount_to_order( $order_id,$posted ) {

            if( !is_account_page() ) {
                $functions = YITH_Dynamic_Pricing_Payments_Methods()->functions;
                $user = wp_get_current_user();
                $order = wc_get_order($order_id);
                $payment_method = yit_get_prop($order, '_payment_method', true);
                $amount = $functions->yith_wcdppm_get_amount($payment_method, $user->roles, true);
                $ordertotal = $order->get_total() + wc_format_decimal($amount[0]) + wc_format_decimal($amount[1]);
                if ($ordertotal > 0) {
                    $order->set_total($order->get_total() + $amount[0] + $amount[1]);
                } else {
                    $order->set_total(0);
                }
                if (is_callable(array($order, 'save'))) {
                    $order->save();
                }
                yit_save_prop($order, '_yith_wcdppm_amount', $amount);
            }
        }

        /**
         * Override tax field in Order received
         * @access public
         * @since 1.0.0
         * @author   Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
         */
        public function override_tax_field($total_row,$order,$tax_display='') {

            if( is_checkout() && !is_account_page() && !did_action('woocommerce_checkout_process') ) {

                $amount = yit_get_prop($order,'yith_wcdppm_amount',true);
                /*$total_order = wc_tax_enabled() ? $order->get_total() - $amount[1]: $order->get_total() + $amount[1];*/
                $total_order = $order->get_total();
                if( $total_order > 0 ) {
                    $total_row['order_total']['value'] = wc_price($total_order);

                } else {
                    $total_row['order_total']['value'] = wc_price(0);
                }

                //yit_save_prop($order,'total',$total_order);

                if( wc_tax_enabled() ) {
                    if('itemized' === get_option( 'woocommerce_tax_total_display' )) {
                        $total_tax = $order->get_tax_totals();
                        if( $total_tax && 'excl' === get_option( 'woocommerce_tax_display_shop' ) ) {
                            $fist_tax = array_shift($total_tax);
	                        $display_tax = $fist_tax->amount /*+ $amount[ 1 ]*/;
                            $total_row['tax-1']['value'] = ($display_tax > 0 ) ? wc_price($display_tax) : wc_price(0);
                            $total_row['tax-1']['label'] = (isset($fist_tax->label)) ? $fist_tax->label : '';
                        }


                    } else {

                        $total_tax = $order->get_total_tax();
                        if( $total_tax && 'excl' === get_option( 'woocommerce_tax_display_shop' )  ) {
                            $display_tax = $total_tax + $amount[ 1 ];
                            $total_row['tax']['value'] = ($display_tax > 0 ) ? wc_price($display_tax) : wc_price(0);
                        }
                    }
                }
            }
            return $total_row;
        }
        /**
         * Override total in top page on Order received page
         * @access public
         * @since 1.0.0
         * @author   Carlos Rodríguez <carlos.rodriguez@yourinspiration.it>
         */
        public function override_total($formatted_total, $order, $tax_display='', $display_refunded='') {

            if( is_checkout() && !is_account_page() && !did_action('woocommerce_checkout_process') ) {
                $amount = yit_get_prop($order,'yith_wcdppm_amount',true);
                $total_order = $order->get_total();
                if( $total_order > 0 ) {
                    $formatted_total = wc_price($total_order);

                } else {
                    $formatted_total = wc_price(0);
                }
            }

            return $formatted_total;
        }
    }
}




/**
 * Unique access to instance of YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium class
 *
 * @return \YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium
 * @since 1.0.0
 */

function YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium() {

    return YITH_Dynamic_Pricing_Payments_Methods_Frontend_Premium::get_instance();
}